#include "httpd.h"
#include "http_config.h"
#include "http_protocol.h"
#include "ap_config.h"

#include "blosxom.h"

static void *mb_get_top_plugin()
{
    return (void *)blosxom_top_plugin;
}

static int mb_is_blosxom_entry_file(blosxom_context_rec *ctx, void *entry)
{
    blosxom_conf_rec *conf;
    const char *path = entry;
    char *ext;

    ext = strrchr(path, '.');
    if (ext == NULL || ext <= path
        || strcmp(conf->file_extension, ext) != 0) {
        return 0;
    }
    return 1;
}

static char *find_date(array_header *list, int *start)
{
    int i;
    char **array = (char **)list->elts;
    char *token = NULL;

    for (i = *start; i < list->nelts; i++) {
        if (array[i] != NULL) {
            token = array[i];
            break;
        }
    }
    *start = ++i;
    return token;
}

blosxom_context_rec *mb_create_context(request_rec *r)
{
    char *pi, *token, *flavour;
    blosxom_context_rec *ctx;
    array_header *pi_list;
    char **pi_array;
    int i, last;

    pi_list = ap_make_array(r->pool, 3, sizeof(char *));
    pi = ap_pstrdup(r->pool, r->path_info);
    ctx = ap_pcalloc(r->pool, sizeof(blosxom_context_rec));
    ctx->request_rec = r;
    ctx->pool = r->pool;
    ctx->get_top_plugin = mb_get_top_plugin;
    ctx->is_entry = mb_is_blosxom_entry_file;
    ctx->bind_string = ap_make_table(r->pool, 10);
    ctx->path_info = "";

    (void)ap_getword(r->pool, (const char **)&pi, '/');
    while (*pi && (token = ap_getword(r->pool, (const char **)&pi, '/'))) {
        *(char **)ap_push_array(pi_list) = token;
    }
    pi_array = (char **)pi_list->elts;

    /* grep real path_info */
    for (i = 0; i < pi_list->nelts; i++) {
        char *token = pi_array[i];
        if (!mb_is_alpha_string(token) || mb_has_dot_char(token))
            continue;
        ctx->path_info = ap_pstrcat(r->pool, ctx->path_info, "/", token, NULL);
        pi_array[i] = NULL;
    }

    /* each flavour */
    flavour = pi_array[pi_list->nelts - 1];
    if (mb_has_dot_char(flavour)) {
         char *n, *f_dot;
         f_dot = strchr(flavour, '.');
         n = ap_pstrndup(r->pool, flavour, f_dot - flavour);
         ctx->flavour = ap_pstrdup(r->pool, f_dot + 1);
         if (strcmp(n, "index") != 0) {
             ctx->path_info = ap_pstrcat(r->pool, ctx->path_info,
                                         "/", n, ".", ctx->flavour, NULL);
         }
         pi_array[pi_list->nelts - 1] = NULL;
    }
    else {
        ctx->flavour = MB_DEFAULT_FLAVOUR;
    }

    /* set date info */
    last = 0;
    ctx->year  = find_date(pi_list, &last);
    ctx->month = find_date(pi_list, &last);
    ctx->date  = find_date(pi_list, &last);
    if (ctx->month == NULL) {
        ctx->num = NULL;
        return ctx;
    }
    if (mb_is_digit_string(ctx->month)) {
        ctx->num = ctx->month;
    }
    else {
        ctx->num = ctx->month;
    }
    return ctx;
}



