#ifndef BLOSXOM_H
#define BLOSXOM_H 1


#define MOD_BLOSXOM_STRING_VERSION "mod_blosxom/2.0pre1"

module MODULE_VAR_EXPORT blosxom_module;


/*
 * Default variables
 */
static const char MB_BLOG_TITLE[]       = "My Blosxom";
static const char MB_BLOG_DESCRIPTION[] = "Yet another Blosxom blog.";
static const char MB_BLOG_LANGUAGE[]    = "en";
static const char MB_DATADIR[]          = "/Library/WebServer/Documents/blosxom";
static const char MB_URL[]              = "";
static const int MB_DEPTH               = 0;
static const int MB_NUM_ENTRIES         = 40;
static const char MB_FILE_EXTENSION[]   = "txt";
static const char MB_DEFAULT_FLAVOUR[]  = "html";

/* Plugins */
static const char MB_PLUGIN_DIR[]       = "";
static const char MB_PLUGIN_STATE_DIR[] = "state";

typedef struct {
    const char *title;
    const char *description;
    const char *language;
    const char *datadir;
    const char *url;
    int depth;
    int num_entries;
    const char *file_extension;
    const char *default_flavour;
    const char *plugin_dir;
    const char *plugin_state_dir;
    array_header *plugin_name;
} blosxom_conf_rec;


typedef struct blosxom_context_rec {
    request_rec *request_rec;
    pool *pool;
    char *path_info;
    char *year;
    char *month;
    char *date;
    char *num;
    const char *flavour;
    void *(*get_top_plugin)();
    int (*is_entry)(struct blosxom_context_rec *ctx, void *entry);
    const char *(*get_template)(struct blosxom_context_rec *ctx,
                                const char *chunk);
    const char *(*interpolate)(struct blosxom_context_rec *ctx,
                               const char *txt);
    array_header *entries;  /* blosxom_entry's list */
    table *bind_string;
} blosxom_context_rec;

typedef struct blosxom_entry_rec {
    void *source;
    time_t mtime;
} blosxom_entry;

#define BLOSXOM_PLUGIN_STUFF  __FILE__, \
                              1, \
                              NULL

typedef struct blosxom_plugin_rec {
    const char *name;
    int enable;
    struct blosxom_plugin_rec *prev;
    int (*start)(blosxom_context_rec *ctx);
    const char *(*template)(blosxom_context_rec *ctx, const char *chunk);
    int (*entries)(blosxom_context_rec *ctx);
    int (*filter)(blosxom_context_rec *ctx);
    int (*end)(blosxom_context_rec *ctx);
    int (*skip)(blosxom_context_rec *ctx);
    const char *(*interpolate)(blosxom_context_rec *ctx, const char *txt);
    int (*head)(blosxom_context_rec *ctx);
    int (*sort)(blosxom_context_rec *ctx);
    int (*date)(blosxom_context_rec *ctx);
    int (*story)(blosxom_context_rec *ctx);
    int (*foot)(blosxom_context_rec *ctx);
    int (*last)(blosxom_context_rec *ctx);
} blosxom_plugin;

typedef struct {
    char *flavour;
    char *chunk;
    const char *body;
} template_rec;

typedef struct {
    const char *head;
    array_header *story;
    const char *foot;
} blosxom_response_body;

blosxom_context_rec *mb_create_context(request_rec *r);
int mb_is_alpha_string(char *str);
int mb_is_digit_string(char *str);
int mb_has_dot_char(char *str);

blosxom_plugin core_blosxom_plugin;
blosxom_plugin *blosxom_top_plugin;


#endif /* !BLOSXOM_H */
